//******************************************************************************
// FILE:       	demo.c
//              
// AUTHOR:     	CC Cox - SynchroSystems
//              
// PROJECT:    	SynchroChron
//              
// DESCRIPTION:	SynchroChron LCD demonstration functions
//
// HISTORY:
//		$Log: demo.c,v $
//		Revision 1.4  2007-08-20 16:59:05-04  chuck
//		Release
//
//		Revision 1.3  2007-08-20 16:39:10-04  chuck
//		Release
//
//		Revision 1.2  2006-01-26 12:17:11-05  chuck
//		Release candidate
//
//		Revision 1.1 2006-01-13 chuck
//		Code review changes
//
//		Revision 1.0 2005-11-06 chuck
//		Initial revision
//
// Copyright (c) 2006 SynchroSystems Inc
// Use subject to terms of License Agreement at http://synchro.com/policies.html
//******************************************************************************

//------------------------------------------------------------------------------
// include files
//------------------------------------------------------------------------------
#include "demo.h"							// project definitions

//******************************************************************************
// PRIVATE DECLARATIONS
//******************************************************************************

//------------------------------------------------------------------------------
// constants
//------------------------------------------------------------------------------
#define FLASH_TICKS		15					// ticks per flash cycle
#define SCROLL_TICKS	20					// ticks per scroll cycle
#define SEGS_TICKS		10					// ticks per demo_segs() step
#define AREAS_TICKS		100					// ticks per demo_areas() step
#define LIGHT_TICKS		1					// ticks per demo_light() step
#define LIGHT_STEP		2					// demo_light() brightness step size
#define MAX_BRIGHT		328					// maximum brightness setting
#define MIN_BRIGHT		4					// minimum brightness to avoid sync glitch
#define TIMING_TICKS	500					// ticks at end of demo_timing() cycle
// initial spinning dial mask
#define SPIN_MASK		LCD_HAND(0) | LCD_HAND(1) | LCD_HAND(2) | \
						LCD_HAND(6) | LCD_HAND(7) | LCD_HAND(8) ;			

//------------------------------------------------------------------------------
// macros
//------------------------------------------------------------------------------
#define LIGHT_OFF		(TACCTL1 = OUT | OUTMOD_0)		// output high = LED off
#define LIGHT_ON		(TACCTL1 = OUTMOD_0)			// output low = LED on
#define LIGHT_PWM		(TACCTL1 = OUTMOD_3)			// PWM set/reset mode
// set backlight brightness: CNT is LED on-time in ticks
#define LIGHT_SET(CNT)	sleep_ticks(LIGHT_TICKS); (TACCR1 = CNT)

//------------------------------------------------------------------------------
// ROM data
//------------------------------------------------------------------------------
static const UINT8 	ID[] 			= "$Id: demo.c,v 1.4 2007-08-20 16:59:05-04 chuck Exp chuck $";	// RCS ID string
static const UINT16	SCROLL_DELAY	= 10;		// scrolling delay (ticks)

//------------------------------------------------------------------------------
// functions
//------------------------------------------------------------------------------
static void demo_areas(void);
static void demo_segs(void);
static void demo_font(void);
static void demo_light(void);
static void demo_timing(void);
static void sleep_ticks(UINT16 ticks);
static void sleep_scroll(void);
static void sleep_scroll_spin(void);

//******************************************************************************
// PUBLIC FUNCTIONS
//******************************************************************************

//------------------------------------------------------------------------------
//  FUNCTION:   	Demo
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	demonstrate LCD features
//------------------------------------------------------------------------------
void Demo(void)
{
	for (;;)
	{
		// introduction
		LcdClear();
		LcdShowScroll("****** SYNCHROCHRON LCD ******");
		sleep_scroll_spin();
		// sequence through demos
		demo_segs();
		demo_areas();
		demo_font();
		demo_light();
		demo_timing();
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	DemoTimer
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	demo timer tick, called by interrupt handler
//------------------------------------------------------------------------------
void DemoTimer(void)
{
 	static INT8 flash_timer = FLASH_TICKS;		// flash delay timer
	static INT8 scroll_timer = SCROLL_TICKS; 	// scroll delay timer

	BITCLR(LED_PORT, LED_BIT);					// LED on

  	if (--flash_timer == 0)
	{
		// time to flash
		flash_timer = FLASH_TICKS;				// reset timer
		LcdFlashTimer();						// flash display
	}
   	if (--scroll_timer == 0)
	{
		// time to scroll
		scroll_timer = SCROLL_TICKS;			// reset timer
		LcdScrollTimer();						// scroll display
	}
}

//******************************************************************************
// PRIVATE FUNCTIONS
//******************************************************************************

//------------------------------------------------------------------------------
//  FUNCTION:   	demo_segs
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	demonstrate LCD segments
//------------------------------------------------------------------------------
static void demo_segs(void)
{
	UINT16 segs;								// segment mask

	// intro	
	LcdClear();
	LcdShowScroll("160 SEGMENTS");
	sleep_scroll();
	// turn all segments on
	for (segs = 1; segs <= LCD_ALPHA_ALL_SEGS; segs = (segs << 1) | 1)
	{
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 1, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 2, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 3, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 4, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 5, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  0, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  1, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  2, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  3, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_MODE_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_MODE_AREA, 1, STEADY);
		LcdShowSegs(segs & LCD_DIAL_ALL_SEGS,  LCD_DIAL_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_PUNCT_ALL_SEGS, LCD_PRIM_PUNCT_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_PUNCT_ALL_SEGS, LCD_PRIM_PUNCT_AREA, 1, STEADY);
		LcdShowSegs(segs & LCD_PUNCT_ALL_SEGS, LCD_SEC_PUNCT_AREA,  0, STEADY);
		LcdShowSegs(segs & LCD_ICON_ALL_SEGS,  LCD_ICON_AREA, LCD_ALARM_POS, STEADY);
		LcdShowSegs(segs & LCD_ICON_ALL_SEGS,  LCD_ICON_AREA, LCD_BELL_POS, STEADY);
		sleep_ticks(SEGS_TICKS);
	}
	// turn all segments off
	for (segs = LCD_ALPHA_ALL_SEGS; segs > 0; segs <<= 1)
	{
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 1, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 2, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 3, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 4, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_PRIM_AREA, 5, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  0, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  1, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  2, STEADY);
		LcdShowSegs(segs & LCD_NUMER_ALL_SEGS, LCD_SEC_AREA,  3, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_MODE_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_ALPHA_ALL_SEGS, LCD_MODE_AREA, 1, STEADY);
		LcdShowSegs(segs & LCD_DIAL_ALL_SEGS,  LCD_DIAL_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_PUNCT_ALL_SEGS, LCD_PRIM_PUNCT_AREA, 0, STEADY);
		LcdShowSegs(segs & LCD_PUNCT_ALL_SEGS, LCD_PRIM_PUNCT_AREA, 1, STEADY);
		LcdShowSegs(segs & LCD_PUNCT_ALL_SEGS, LCD_SEC_PUNCT_AREA,  0, STEADY);
		LcdShowSegs(segs & LCD_ICON_ALL_SEGS,  LCD_ICON_AREA, LCD_ALARM_POS, STEADY);
		LcdShowSegs(segs & LCD_ICON_ALL_SEGS,  LCD_ICON_AREA, LCD_BELL_POS, STEADY);
		sleep_ticks(SEGS_TICKS);
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	demo_areas
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	demonstrate LCD display areas
//------------------------------------------------------------------------------
static void demo_areas(void)
{
	// primary
	LcdClear();
	LcdShowScroll("6 LARGE PRIMARY CHARACTERS");
	sleep_scroll();
	LcdShowStr("######", LCD_PRIM_AREA, 0, FLASH);
	LcdShowChar(':', LCD_PRIM_PUNCT_AREA, 0, STEADY);
	LcdShowChar(':', LCD_PRIM_PUNCT_AREA, 1, STEADY);
	sleep_ticks(3 * AREAS_TICKS);
	// secondary
	LcdClear();
	LcdShowScroll("4 SMALL SECONDARY DIGITS");
	LcdShowStr("8888", LCD_SEC_AREA, 0, FLASH);
	LcdShowChar(':', LCD_SEC_PUNCT_AREA, 0, STEADY);
	sleep_scroll();
	sleep_ticks(AREAS_TICKS);
	// mode
	LcdClear();
	LcdShowScroll("2 SMALL MODE CHARACTERS");
	LcdShowStr("##", LCD_MODE_AREA, 0, FLASH);
	sleep_scroll();
	sleep_ticks(AREAS_TICKS);
	// dial
	LcdClear();
	LcdShowScroll("12 DIAL HANDS");
	sleep_scroll_spin();
	LcdShowSegs(LCD_DIAL_ALL_SEGS, LCD_DIAL_AREA, 0, FLASH);
	sleep_ticks(AREAS_TICKS);
	// icons
	LcdClear();
	LcdShowScroll("2 ICONS");
	LcdShowSegs(LCD_ICON_ON, LCD_ICON_AREA, LCD_ALARM_POS, FLASH);
	sleep_ticks(FLASH_TICKS);
	LcdShowSegs(LCD_ICON_ON, LCD_ICON_AREA, LCD_BELL_POS, FLASH);
	sleep_scroll();
	sleep_ticks(AREAS_TICKS);
}

//------------------------------------------------------------------------------
//  FUNCTION:   	demo_font
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	demonstrate the font
//------------------------------------------------------------------------------
static void demo_font(void)
{
	UINT8 alpha = 'A';
	UINT8 num = '0';
	
	// intro
	LcdClear();
	LcdShowScroll("ALPHANUMERIC FONT");
	sleep_scroll();
	// show font in alphanumeric & numeric areas
	LcdShowScroll("ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890+-/\\<>[]{}$%&*?@_|");
	while (LcdIsScrolling())
	{
		LcdShowChar(alpha, LCD_MODE_AREA, 0, STEADY);
		LcdShowChar(alpha, LCD_MODE_AREA, 1, STEADY);
		if (++alpha > 'Z') 
		{
			alpha = 'A';			// wrap-around
		}
		LcdShowChar(num, LCD_SEC_AREA, 0, STEADY);
		LcdShowChar(num, LCD_SEC_AREA, 1, STEADY);
		LcdShowChar(num, LCD_SEC_AREA, 2, STEADY);
		LcdShowChar(num, LCD_SEC_AREA, 3, STEADY);
		if (++num > '9') 
		{	
			num = '0';				// wrap-around
		}
		sleep_ticks(SCROLL_TICKS);
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	demo_light
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	demonstrate the backlight
//------------------------------------------------------------------------------
static void demo_light(void)
{
	UINT32 bright;						// brightness
	UINT8 ma;							// milliamps
	UINT8 graph;						// percentage graph
	UINT16 dial = 1;					// dial graph mask
		
	// intro	
    LIGHT_ON;							// backlight on
	LcdClear();							// clear display
	LcdShowScroll("LED BACKLIGHT");		// scroll message
	sleep_scroll();						// wait for scrolling to complete
    LIGHT_OFF;							// backlight off
	
	// dimming demo
	// activate PWM
	LIGHT_SET(0);						// 0% duty cycle
    LIGHT_PWM;							// enable backlight PWM mode
	// initialize display
	LcdShowStr("BL", LCD_MODE_AREA, 0, STEADY);
	LcdShowChar(' ', LCD_PRIM_PUNCT_AREA, 0, STEADY);
	LcdShowChar(' ', LCD_PRIM_PUNCT_AREA, 1, STEADY);
	LcdShowStr("  00mA", LCD_PRIM_AREA, 0, STEADY);
	// ramp up backlight
	for (bright = MIN_BRIGHT; bright < MAX_BRIGHT; bright += LIGHT_STEP)
	{
		// set duty cycle
		LIGHT_SET(bright);				
		// display amperage
		ma = 30 * bright / MAX_BRIGHT;
		LcdShowUDec2(ma, LCD_PRIM_AREA, 2, STEADY, LEAD_ZERO);
		// display graph
		dial = (1 << (12 * bright / MAX_BRIGHT)) - 1;
		LcdShowSegs(dial, LCD_DIAL_AREA, 0, STEADY);
	}
	// pause at max brightness
	bright = MAX_BRIGHT;
	// set duty cycle
	LIGHT_SET(bright);				
	// display amperage
	ma = 30 * bright / MAX_BRIGHT;
	LcdShowUDec2(ma, LCD_PRIM_AREA, 2, STEADY, LEAD_ZERO);
	// display graph
	dial = (1 << (12 * bright / MAX_BRIGHT)) - 1;
	LcdShowSegs(dial, LCD_DIAL_AREA, 0, STEADY);
	// pause
	sleep_ticks(100 * LIGHT_TICKS);
	// ramp down backlight
	for (bright = MAX_BRIGHT; bright >= MIN_BRIGHT; bright -= LIGHT_STEP)
	{
		// set duty cycle
		LIGHT_SET(bright);				
		// display amperage
		ma = 30 * bright / MAX_BRIGHT;
		LcdShowUDec2(ma, LCD_PRIM_AREA, 2, STEADY, LEAD_ZERO);
		// display graph
		dial = (1 << (12 * bright / MAX_BRIGHT)) - 1;
		LcdShowSegs(dial, LCD_DIAL_AREA, 0, STEADY);
	}
	// disable PWM
    LIGHT_OFF;							// backlight off
    LIGHT_SET(0);						// 0% duty cycle
	sleep_ticks(100 * LIGHT_TICKS);
}

//------------------------------------------------------------------------------
//  FUNCTION:   	demo_timing
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	demonstrate a simple countdown timer
//------------------------------------------------------------------------------
static void demo_timing(void)
{
	UINT8 min = 0;						// minutes
	UINT8 sec = 11;						// seconds
	UINT8 hund = 0;						// hundredths of a second
	UINT8 cnt;							// loop counter

	// intro	
	LcdClear();
	LcdShowScroll("IDEAL FOR TIMING");
	sleep_scroll();
	// initialize display
	LcdShowStr("TR", LCD_MODE_AREA, 0, STEADY);
	LcdShowChar('-', LCD_SEC_PUNCT_AREA, 0, STEADY);
	LcdShowStr("1122", LCD_SEC_AREA, 0, STEADY);
	LcdShowChar(':', LCD_PRIM_PUNCT_AREA, 0, STEADY);
	LcdShowChar('.', LCD_PRIM_PUNCT_AREA, 1, STEADY);
	// countdown to zero
	while ((sec | hund) != 0)
	{
		if (hund != 0)
		{
			hund--;
		}
		else
		{
			hund = 99;
			sec--;
		}
		LcdShowUDec2(min,  LCD_PRIM_AREA, 0, STEADY, LEAD_ZERO);
		LcdShowUDec2(sec,  LCD_PRIM_AREA, 2, STEADY, LEAD_ZERO);
		LcdShowUDec2(hund, LCD_PRIM_AREA, 4, STEADY, LEAD_ZERO);
		LcdShowDial100(hund, STEADY);
		sleep_ticks(1);
	}
	// zero reached
	LcdShowUDec2(min,  LCD_PRIM_AREA, 0, FLASH, LEAD_ZERO);
	LcdShowUDec2(sec,  LCD_PRIM_AREA, 2, FLASH, LEAD_ZERO);
	LcdShowUDec2(hund, LCD_PRIM_AREA, 4, FLASH, LEAD_ZERO);
	LcdShowSegs(LCD_ICON_ON, LCD_ICON_AREA, LCD_ALARM_POS, FLASH);
//	sleep_ticks(TIMING_TICKS);
	// flash backlight
	for (cnt = 0; cnt < 10; cnt++)
	{
    	LIGHT_ON;						// backlight on
		sleep_ticks(5);
	    LIGHT_OFF;						// backlight off
		sleep_ticks(45);
	}
}

//==============================================================================
// sleep functions
//==============================================================================

//------------------------------------------------------------------------------
//  FUNCTION:   	sleep_ticks
//              
//  PARAMETERS: 	ticks		number of timer ticks to sleep
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	sleep in low power mode
//------------------------------------------------------------------------------
static void sleep_ticks(UINT16 ticks)
{
	while (ticks-- != 0)
	{
		// sleep until next timer interrupt
		BITSET(LED_PORT, LED_BIT);			// LED off
		LO_PWR;								// low power mode
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	sleep_scroll
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	sleep until for LCD stops scrolling
//------------------------------------------------------------------------------
static void sleep_scroll(void)
{
	while (LcdIsScrolling())
	{
		sleep_ticks(SCROLL_DELAY);
	}
}

//------------------------------------------------------------------------------
//  FUNCTION:   	sleep_scroll_spin
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	spin dial and sleep until for LCD stops scrolling
//------------------------------------------------------------------------------
static void sleep_scroll_spin(void)
{
	static UINT16 dial = SPIN_MASK;			// dial hand mask
	
	while (LcdIsScrolling())
	{
		LcdShowDialCW(&dial, STEADY);		// spin dial hands
		sleep_ticks(SCROLL_DELAY);
	}
}

