//******************************************************************************
// FILE:       	init.c
//              
// AUTHOR:     	CC Cox - SynchroSystems
//              
// PROJECT:    	SynchroChron
//              
// DESCRIPTION:	Device initialization functions
//
// HISTORY:
//		$Log: init.c,v $
//		Revision 1.5  2007-08-20 16:39:11-04  chuck
//		Release
//
//		Revision 1.4  2006-01-26 12:17:14-05  chuck
//		Release candidate
//
//		Revision 1.3 2006-01-13 chuck
//		Code review changes
//
//		Revision 1.0 2005-11-14 chuck
//		Initial revision
//
// Copyright (c) 2006 SynchroSystems Inc
// Use subject to terms of License Agreement at http://synchro.com/policies.html
//******************************************************************************

//------------------------------------------------------------------------------
// include files
//------------------------------------------------------------------------------
#include "demo.h"						// project definitions

//******************************************************************************
// PRIVATE DECLARATIONS
//******************************************************************************

//------------------------------------------------------------------------------
// ROM data
//------------------------------------------------------------------------------
static const UINT8 ID[] = "$Id: init.c,v 1.5 2007-08-20 16:39:11-04 chuck Exp chuck $";		// RCS ID string
// timer ticks per 100Hz heartbeat
// 32768Hz/328 = 99.90Hz (10.01mS)
static const UINT16 HEART_TICKS = 328;		

//------------------------------------------------------------------------------
// functions
//------------------------------------------------------------------------------
static void init_clocks(void);			// initialize FLL clock
static void init_gpio(void);			// initialize general-purpose I/O
static void init_basic_timer(void);		// initialize basic timer
static void init_timer_a(void);			// initialize timer A

//******************************************************************************
// PUBLIC FUNCTIONS
//******************************************************************************

//------------------------------------------------------------------------------
//  FUNCTION:   	Init
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	device initialization
//------------------------------------------------------------------------------
void Init(void)
{
    WDTCTL = WDTPW | WDTHOLD;		// disable WDT					
    init_clocks();					// initialize FLL clock
	init_gpio();					// initialize general-purpose I/O
	init_basic_timer();				// initialize basic timer
	init_timer_a();					// initialize timer A
	LcdInit();						// initialize LCD driver						
    EINT(); 						// enable global interrupts
}

//******************************************************************************
// PRIVATE FUNCTIONS
//******************************************************************************

//------------------------------------------------------------------------------
//  FUNCTION:   	init_clocks
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	initialize FLL clock
//
// 					XT1:	32768Hz @ 1pF
// 					XT2:	off
// 					FLL:	1MHz, modulated
// 					ACLK:	32768Hz
// 					ACLK/n:	32768Hz
// 					MCLK:	1MHz
// 					SMCLK:	off
//------------------------------------------------------------------------------
static void init_clocks(void)
{
    SCFQCTL = SCFQ_1M; 				// system clock frequency control						
    SCFI0 = FLLD_2;					// system clock frequency integrator 0
    SCFI1 = 0;						// system clock frequency integrator 1
    FLL_CTL0 = XCAP0PF;				// FLL+ Control 0
    FLL_CTL1 = SMCLKOFF | XT2OFF;	// FLL+ Control 1				
}

//------------------------------------------------------------------------------
//  FUNCTION:   	init_gpio
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	initialize general-purpose I/O
//------------------------------------------------------------------------------
static void init_gpio(void)
{
	// Port1: LED output
    P1OUT = 0x00;
    P1DIR = 0x05;
    P1IES = 0x00;
    P1IE  = 0x00;
    P1SEL = 0x04;
	// Port4: LCD peripheral
    P4OUT = 0x00;
    P4DIR = 0x00;
    P4SEL = 0xFC;
	// Port5: LCD peripheral
    P5OUT = 0x00;
    P5DIR = 0x00;
    P5SEL = 0xFF;
}

//------------------------------------------------------------------------------
//  FUNCTION:   	init_basic_timer
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	initialize basic timer1 for LCD timing
//
// 					fLCD = fACLK/64 = 512Hz
// 					LCD frame rate = fLCD/8 = 64Hz
//------------------------------------------------------------------------------
static void init_basic_timer(void)
{
    BTCTL = BTHOLD_CNT1_2;				// stop timer
	// reset counters
    BTCNT1 = 0;							
    BTCNT2 = 0;							
	// start timer
    BTCTL = BT_fLCD_DIV64 | BT_fCLK2_ACLK_DIV256 | BT_fCLK2_DIV256;
}

//------------------------------------------------------------------------------
//  FUNCTION:   	init_timer_a
//              
//  PARAMETERS: 	none
//              
//  RETURNS:    	nothing
//              
//  DESCRIPTION:	initialize timer_A for 100Hz heartbeat interrupt
//
// 					mode = compare
// 					clock = ACLK/1 = 32768Hz
// 					interrupt = 32768Hz/328 = 99.90Hz (10.01mS)
//------------------------------------------------------------------------------
static void init_timer_a(void)
{
    TACTL = 0x0000; 					// stop timer
    TACCTL0 = CCIE;						// enable TACCR0 interrupt
    TACCR0 = HEART_TICKS;				// compare value 
    TACCTL1 = OUT | OUTMOD_0;			// output high = LED off
    TACCR1 = 0x0000;
    TACCTL2 = 0x0000;
    TACCR2 = 0x0000;
    TAR = 0x0000;
    TACTL = TASSEL_1 | MC_1 | TACLR;	// start timer
}
